void request_param_IS() {

  // Текущие параметры инвертора
  int U_grid ;        // Напряжение входной сети
  int F_grid ;        // Частота входной сети
  int U_out ;         // Выходное напряжение
  int S_out ;         // Выходня мощность полная (ВА)
  int P_out ;         // Выходная мощность активная (Вт)
  int Load ;          // Нагрузка инвертора, (%)
  int U_bat ;         // Напряжение АКБ
  int I_charge ;      // Ток заряда АКБ
  int I_discharge ;   // Ток разряда АКБ
  int C_bat ;         // Уровень заряда АКБ (%)
  int T_inv ;         // Температура инвертора
  int P_pv1 ;         // Мощность от СБ
  int P_pv2 ;         // Мощность от СБ
  int U_pv1 ;         // Напряжение СБ
  int U_pv2 ;         // Напряжение СБ

  //*-- Запрос текущих параметров инвертора
  response = sendCmd("^P005GS");                         // Команда запроса текущих параметров

  if ( response != "Err" ) {
    char responseArr[120];
    response.toCharArray(responseArr, 120);              // Преобразование ответа CharArray
    Serial.println(" Data OK");

    //^D1060000,000,2300,500,0006,0006,000,502,000,000,000,000,080,026,000,000,0000,0000,0000,0000,0,0,0,1,2,2,0,0⸮'
    sscanf(responseArr, "^D106%d,%d,%d,%*d,%d,%d,%d,%d,%*d,%*d,%d,%d,%d,%d,%*d,%*d,%d,%d,%d,%d", &U_grid, &F_grid, &U_out, &S_out, &P_out, &Load, &U_bat, &I_discharge, &I_charge, &C_bat, &T_inv, &P_pv1, &P_pv2, &U_pv1, &U_pv2);

    dataToCloud = "&p1=";
    dataToCloud += U_grid * 0.1;
    dataToCloud += "&p2=";
    dataToCloud += F_grid * 0.1;
    dataToCloud += "&p3=";
    dataToCloud += U_out * 0.1;
    dataToCloud += "&p4=";
    dataToCloud += S_out;
    dataToCloud += "&p5=";
    dataToCloud += P_out;
    dataToCloud += "&p6=";
    dataToCloud += Load;
    dataToCloud += "&p7=";
    dataToCloud += U_bat * 0.1;
    dataToCloud += "&p8=";
    dataToCloud += I_discharge;
    dataToCloud += "&p9=";
    dataToCloud += I_charge;
    dataToCloud += "&p10=";
    dataToCloud += C_bat;
    dataToCloud += "&p11=";
    dataToCloud += T_inv;
    dataToCloud += "&p12=";
    dataToCloud += P_pv1;
    dataToCloud += "&p13=";
    dataToCloud += P_pv2;
    dataToCloud += "&p14=";
    dataToCloud += U_pv1 * 0.1;
    dataToCloud += "&p15=";
    dataToCloud += U_pv2 * 0.1;
    //dataToCloud += "&p16=";
    //dataToCloud += Ipvbat;
  }

  else {
    //Serial.println("Err: Data Incorrect");
    //dataToCloud = "";           // *** УБРАЛ Высвечиваем "Нет данных", если инвертор не отвечает или пакет не корректный

    //Мигаем светодиодом
    pinMode(ledPin, OUTPUT);
    digitalWrite(ledPin, LOW);
    timerStop(4000);
    pinMode(ledPin, INPUT);   // Уменьшаем свечение
  }
}


// *** Запрос статуса инвертора ***
void request_status_IS() {

  //*-- Запрос текущего статуса инвертора
  timerStop(2000);

  while (Serial.available()) Serial.read();
  Serial.print("\x5E\x50\x30\x30\x36\x4D\x4F\x44\xDD\xBE\x0D");
  timerStop(1000);

  String result = "Unknown";
  if (Serial.find("^D005")) {
    int Status = Serial.parseInt();

    switch (Status)
    {
      case 0: result = "Power_On_Mode";     break;
      case 1: result = "Standby_Mode";      break;
      case 2: result = "Bypass_mode";       break;
      case 3: result = "Battery_Mode";      break;
      case 4: result = "Fault_Mode";        break;
      case 5: result = "Hybrid_mode";       break;
      default:  result = "Unknown";         break;
    }
  }
  if (debug) {
    Serial.print(" Статус: ");
    Serial.println(result);
  }

  //*-- Подготовка строки для отправки на сервер
  dataToCloud += "&p18=";
  dataToCloud += result;
}


// *** Запрос исторических данных ***
void calc_Watt_IS() {

  //*-- Запрос энергии Тотал --*//
  timerStop(2000);
  response = sendCmd("^P005ET");
  unsigned long Power_Tot;

  if ( response.indexOf("^D011") != -1 ) {
    response = response.substring(5);
    Power_Tot = response.toInt();                         // Энергия Total
    if (debug) {
      Serial.print(" Энергия Тотал: ");
      Serial.println(Power_Tot);
    }
    timerStop(1000);
  }

  //*-- Запрос текущей даты --*//
  char Date [9] = "";

  response = sendCmd("^P004T");                           // Команда запроса текущих параметров
  if ( response != "Err" ) {
    char responseArr[30];
    response.toCharArray(responseArr, 30);
    // Ответ вида: ^D01720160214201314
    sscanf(responseArr, "^D017%8s", &Date);               // Вытаскиваем текущую дату "YYYYMMDD"
    if (debug) {
      Serial.print(" Текущая дата: ");
      Serial.println(String(Date));
    }

    //*-- Запрос дневной генерации --*//
    char cmd_EnDay[30] = "";
    unsigned long Power_Day;
    sprintf(cmd_EnDay, "%s%s", "^P013ED", Date);          // Запрос генерации за текущий день: "^P013ED" + текущая дата

    response = sendCmd((char*)cmd_EnDay);
    if ( response != "Err" ) {
      char responseArr[30];
      response.toCharArray(responseArr, 30);
      // Ответ вида: ^D01100000000⸮⸮
      sscanf(responseArr, "^D011%8d", &Power_Day);        // Вытаскиваем генерацию за 1 день, в Wh
    }
    if (debug) {
      Serial.print(" Дневная сумма: ");
      Serial.println(String(Power_Day));
    }

    // Дописываем пакет для отправки
    dataToCloud += "&p24=";
    dataToCloud += Power_Tot;
    dataToCloud += "&p25=";
    dataToCloud += Power_Day;
  }
}
