// *** Ф-ия принимает команду отправляемую в устройство и возвращает ответ устройства ***
String sendCmd (char *data)
{
  unsigned short crc;
  int n = strlen(data);

  crc = calcrc((char*)data, n);                          // Расчет контрольной суммы
  char crc1 = crc >> 8;                                  // Старший байт
  char crc2 = crc & 0xff;                                // Младший байт
  String cmd = String(data) + crc1 + crc2 + "\x0D";      // Полная команда для устройства

  Serial.print("\x0D");
  timerStop(1000);   
  while (Serial.available()) Serial.read();              // Очистка порта от артефактов
  Serial.print(cmd);
  timerStop(2000);

  int i = 0;
  char buffer[120];

  if (Serial.available()) {
    while ( Serial.available() && i < 119) {
      buffer[i++] = Serial.read();
      yield();
    }
    buffer[i++] = '\0';  //закрываем массив
  }
  if (debug) Serial.println(buffer);                     // Вывод полного ответа в режиме отладки


  // Проверка контрольной суммы ответа
  n = strlen(buffer);
  if (n > 0) {
    crc = calcrc((char*)buffer, n - 1);
    if (crc == 0) {
      return buffer;                                    // Если суммы совпали возвращаем полный ответ инв-ра
    } else {
      Serial.println(" Err: Не совпали CRC");
      return ("Err");                                   // Если суммы не совпали возвращаем "Err"
    }
  } else {
    Serial.println(" No data");
    deviceId = 0;
    return ("Err");                                     // Возвращаем "Err"
  }
}


// *** Ф-ия расчета контрольной суммы ***
int calcrc(char *ptr, int count)
{
  int  crc;
  char i;
  crc = 0;
  while (--count >= 0)
  {
    crc = crc ^ (int) * ptr++ << 8;
    i = 8;
    do
    {
      if (crc & 0x8000)
        crc = crc << 1 ^ 0x1021;
      else
        crc = crc << 1;
    } while (--i);
  }
  return (crc);
}


// *** Функция таймера ***
void timerStop (int ms)
{
  unsigned long NTime_3 = millis();
  unsigned long PTime_3 = NTime_3 ;
  while ( NTime_3 - PTime_3 <= ms ) {
    yield();
    NTime_3 = millis();
  }
}


// *** Функция PROTOCOL DETECT ***
void protocol_version() {
  Serial.println("Поиск поддерживаемых устройств...");

  // 1. - Принадлежность к версии SmartWatt ECO, SILA XXX P/M/Plus
  timerStop(2000);                
  response = sendCmd("QPI");         // Команда "QPI" - Device Protocol ID Inquiry                   
  if ( response.indexOf("(PI") != -1 ) {
    Serial.println("Найден комбиниров. солн. инвертор");
    Serial.println();
    deviceId = 1;
    timerStop(500);
    return;
  } else {
    deviceId = 0;
  }

  // 2. -  Принадлежность к версии Infini Solar
  timerStop(1500);
  response = sendCmd("^P005ID");     // Команда "^P005ID" - Query protocol ID
  if ( response.indexOf("^D") != -1 ) {
    Serial.println("Найден гибридный солн. инвертор");
    Serial.println();
    deviceId = 2;
    return;
  } else {
    deviceId = 0;
  }
  timerStop(500);
}
