/*
   Версия: 40 от 15.09.2022
   Что нового:  Работает с комбинированными: SILA (V)XXXX(P,M,MH), SMartWatt ECO
                и гибридными инверторами: SILA PRO XXXX (ML,MH) SmarWatt Hybrid, сам определяет какой подключен
                В ответе инвертора проверяется контрольная сумма CRC, исключаются ошибки в пакетах
   Версия Arduino IDE 1.8.13 - 1.8.19
   Менеджер плат: ESP8266 версии 2.6.3 - 3.0.2 (Важно, с другими совместимость не гарантирована!!!)
   Плата в настройках IDE: WeMos D1 R1
   Железо: wemos d1 mini + RS232 to TTL converter (MAX3232)
   Подключение: TX ESP <-> TX TTL
                RX ESP <-> RX TTL
                GND ESP <-> GND TTL
                3.3V ESP <-> VCC TTL напряжение в зависимости от чипа TTL конвертора
                3,3V - MAX3232
                5V - MAX232
                D5 <-> Датчик температуры 1 (DS18B20)
   Мониторинг параметров и статуса инвертора. Передача на сервис аналитики http://open-monitoring.online
   Nikolay Semerkov, https://nsk-electro.ru/
   Для связи с автором: semerkov@mail.ru
*/

//*-- Строки требующие редактирования в комментариях отмечены (!)
#include <ESP8266WiFi.h>                         // Точно работает с версиями 2.6.3 - 3.0.2, с остальным не гарантированно
#include <string.h>                              // Идет в пакете, установки не требует
#include <OneWire.h>                             // Установить через менеджер библиотек (2.3.7)

//*-- Индивидуальные настройки
const char* ssid     = "XXXXXXXXXXXXX";          // Название точки доступа     (!)
const char* password = "XXXXXXXXX";              // Пароль                     (!)

//*-- Параметры полученные при регистрации на http://open-monitoring.online
const char* host = "open-monitoring.online";     // Адрес сервера (не менять)
const char* streamId   = "XXXX";                 // ID                         (!)
const char* privateKey = "XXXXXX";               // Код доступа                (!)
unsigned long send_interval = 61000;             // Периодичность отправки пакетов на сервер
                                                 // (не менее 60 сек)
boolean sendTemp = false;                        // Использовать датчики температуры "false" - нет, "true" - да

//*-- Служебные перемнные
boolean debug = false;                           // Режим отладки (протоколирует работу в порт): trui/false
#define ledPin   2                               // Светодиод
int deviceId = 0;                                // Тип инвертора (
//                                                      0 = Не известен/нет связи
//                                                      1 = P_M_Plus,
//                                                      2 = Infini Solar,
//                                                      3 = Контроллер ??? )

String dataToCloud = "";                  // Строка параметров устройства для отправки в open-monitoring.online
String response = "";

//*-- Текущие параметры инвертора (используются в расчете накопленной энергии)
double U_bat_dec;                         // Напряжение АКБ в десятичной форме
int I_pv_bat ;                            // Ток заряда АКБ от СБ
float Power_sum = 0 ;                     // Переменная, хранящая энергию до ее отправки на сервер и
//                                           получения подтверждения о ее принятии


unsigned long previousMillis = 0;
boolean flag_1Send = true; // Флаг первой посылки

OneWire sens1(14);                        // № пина подключения датч. тем-ры 1 (D5 - обознач. на плате ESP)
//OneWire sens2(12);                      // № пина подключения датч. тем-ры 2 (D6 - обознач. на плате ESP)

OneWire * psens1 = &sens1;
//OneWire * psens2 = &sens2;

//*---------------- INICIALIZATION ------------------*//
void setup() {
  Serial.begin(2400);                     // Скорость UART для работы с инвертором
  delay(10);

  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}


//*------------------ MAIN CYCLE --------------------*//
void loop() {

  if (deviceId == 0) protocol_version();  // Поиск устройства, если deviceId == 0
  switch (deviceId)                       // Задачи на выполнение при каждой итерации
    {
      case 0: break;
      case 1: request_param();            // Запрашиваем параметры инвертора
              calc_Watt();                // Рссчитываем выработанную энергию               
              break;
      case 2: request_param_IS();
              break;
    }
    timerStop(2500);

  unsigned long currentMillis = millis();
  if ( currentMillis - previousMillis >= send_interval || flag_1Send == true ) {
    previousMillis = currentMillis;
    flag_1Send = false;                   // Опускаем флаг первой отправки

    switch (deviceId)                     // Задачи на выполнение при каждой отправке
    {
      case 0:                           break;
      case 1: summ_Watt();
        request_status();               break;    // Запрос статуса комби инверторов
      case 2: request_status_IS();
        calc_Watt_IS();                 break;    // Запрос статуса гибридов
    }
    data_send();                          // Отправка данных на сервер
  }
}
